/* ZX81 - Emulator

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>
#include <zlib.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>

#include "global.h"
#include "psp_fmgr.h"

   ZX81_type ZX81;

   int psp_screenshot_mode = 0;

//LUDO:
int
zx81_save_configuration(void)
{
  char  chFileName[MAX_PATH + 1];
  FILE* FileDesc;
  int   error = 0;

  strncpy(chFileName, ZX81.zx81_home_dir, sizeof(chFileName)-10);
  strcat(chFileName, "/pspzx81.cfg");

  FileDesc = fopen(chFileName, "w");
  if (FileDesc != (FILE *)0 ) {

    fprintf(FileDesc, "psp_cpu_clock=%d\n"      , ZX81.psp_cpu_clock);
    fprintf(FileDesc, "psp_reverse_analog=%d\n" , ZX81.psp_reverse_analog);
    fprintf(FileDesc, "zx81_render_mode=%d\n"   , ZX81.zx81_render_mode);
    fprintf(FileDesc, "zx81_speed_limiter=%d\n" , ZX81.zx81_speed_limiter);
    fprintf(FileDesc, "zx81_color_mode=%d\n"    , ZX81.zx81_color_mode);

    fclose(FileDesc);

  } else {
    error = 1;
  }

  return error;
}

int
zx81_parse_configuration(void)
{
  char  chFileName[MAX_PATH + 1];
  char  Buffer[512];
  char *Scan;
  unsigned int Value;
  FILE* FileDesc;

  strncpy(chFileName, ZX81.zx81_home_dir, sizeof(chFileName)-10);
  strcat(chFileName, "/pspzx81.cfg");

  FileDesc = fopen(chFileName, "r");
  if (FileDesc == (FILE *)0 ) return 0;

  while (fgets(Buffer,512, FileDesc) != (char *)0) {

    Scan = strchr(Buffer,'\n');
    if (Scan) *Scan = '\0';
    /* For this #@$% of windows ! */
    Scan = strchr(Buffer,'\r');
    if (Scan) *Scan = '\0';
    if (Buffer[0] == '#') continue;

    Scan = strchr(Buffer,'=');
    if (! Scan) continue;

    *Scan = '\0';
    Value = atoi(Scan+1);

    if (!strcasecmp(Buffer,"psp_cpu_clock"))      ZX81.psp_cpu_clock = Value;
    else
    if (!strcasecmp(Buffer,"psp_reverse_analog")) ZX81.psp_reverse_analog = Value;
    else
    if (!strcasecmp(Buffer,"zx81_render_mode"))   ZX81.zx81_render_mode = Value;
    else
    if (!strcasecmp(Buffer,"zx81_speed_limiter")) ZX81.zx81_speed_limiter = Value;
    else
    if (!strcasecmp(Buffer,"zx81_color_mode"))    ZX81.zx81_color_mode = Value;
  }

  fclose(FileDesc);

  return 0;
}

static int 
loc_load_rom(char *TmpName)
{
# if 0 //LUDO: TO_BE_DONE
  int error = (Atari800_OpenFile(TmpName, 1, 1, FALSE) == AFILE_ERROR);
  return error;
# else
  return 0;
# endif
}

void
zx81_update_save_name(char *Name)
{
  char        TmpFileName[MAX_PATH];
  struct stat aStat;
  int         index;
  char       *SaveName;
  char       *Scan1;
  char       *Scan2;

  SaveName = strrchr(Name,'/');
  if (SaveName != (char *)0) SaveName++;
  else                       SaveName = Name;

  if (!strncasecmp(SaveName, "sav_", 4)) {
    Scan1 = SaveName + 4;
    Scan2 = strrchr(Scan1, '_');
    if (Scan2 && (Scan2[1] >= '0') && (Scan2[1] <= '5')) {
      strncpy(ZX81.zx81_save_name, Scan1, MAX_PATH);
      ZX81.zx81_save_name[Scan2 - Scan1] = '\0';
    } else {
      strncpy(ZX81.zx81_save_name, SaveName, MAX_PATH);
    }
  } else {
    strncpy(ZX81.zx81_save_name, SaveName, MAX_PATH);
  }

  memset(ZX81.zx81_save_used,0,sizeof(ZX81.zx81_save_used));

  if (ZX81.zx81_save_name[0] == '\0') {
    strcpy(ZX81.zx81_save_name,"default");
  }

  for (index = 0; index < ZX81_MAX_SAVE_STATE; index++) {
    snprintf(TmpFileName, MAX_PATH, "%s/save/sav_%s_%d.zst", ZX81.zx81_home_dir, ZX81.zx81_save_name, index);
    if (! stat(TmpFileName, &aStat)) {
      ZX81.zx81_save_used[index] = 1;
    }
  }
}

void
reset_save_name()
{
  zx81_update_save_name("");
}

void
zx81_kbd_load(void)
{
  char        TmpFileName[MAX_PATH + 1];
  struct stat aStat;

  snprintf(TmpFileName, MAX_PATH, "%s/kbd/%s.kbd", ZX81.zx81_home_dir, ZX81.zx81_save_name );
  if (! stat(TmpFileName, &aStat)) {
    psp_kbd_load_mapping(TmpFileName);
  }
}

int
zx81_kbd_save(void)
{
  char TmpFileName[MAX_PATH + 1];
  snprintf(TmpFileName, MAX_PATH, "%s/kbd/%s.kbd", ZX81.zx81_home_dir, ZX81.zx81_save_name );
  return( psp_kbd_save_mapping(TmpFileName) );
}


void
emulator_reset(void)
{
  reset81();
  reset_save_name();
}

int
zx81_snapshot_save_slot(int save_id)
{
  char  FileName[MAX_PATH+1];
  int   error;

  error = 1;

  if (save_id < ZX81_MAX_SAVE_STATE) {
    snprintf(FileName, MAX_PATH, "%s/save/sav_%s_%d.zst", ZX81.zx81_home_dir, ZX81.zx81_save_name, save_id);
    error = zx81_save_state(FileName);
    if (! error) ZX81.zx81_save_used[save_id] = 1;
  }

  return error;
}

int
zx81_snapshot_load_slot(int load_id)
{
  char  FileName[MAX_PATH+1];
  int   error;

  error = 1;

  if (load_id < ZX81_MAX_SAVE_STATE) {
    snprintf(FileName, MAX_PATH, "%s/save/sav_%s_%d.zst", ZX81.zx81_home_dir, ZX81.zx81_save_name, load_id);
    error = zx81_load_state(FileName);
  }
  return error;
}

int
zx81_snapshot_del_slot(int save_id)
{
  char  FileName[MAX_PATH+1];
  int   error;

  error = 1;

  if (save_id < ZX81_MAX_SAVE_STATE) {
    snprintf(FileName, MAX_PATH, "%s/save/sav_%s_%d.zst", ZX81.zx81_home_dir, ZX81.zx81_save_name, save_id);
    error = remove(FileName);
    if (! error) ZX81.zx81_save_used[save_id] = 0;
  }

  return error;
}

void
zx81_initialize(void)
{
  memset(&ZX81, 0, sizeof(ZX81_type));
  getcwd(ZX81.zx81_home_dir,MAX_PATH);

  ZX81.zx81_render_mode    = ZX81_RENDER_NORMAL;
  ZX81.psp_reverse_analog  = 0;
  ZX81.psp_cpu_clock       = 222;
  ZX81.psp_screenshot_id   = 0;
  ZX81.zx81_speed_limiter  = 1;
  ZX81.zx81_color_mode     = ZX81_COLOR_BLUE;

  zx81_parse_configuration();

  zx81_update_save_name("");

  scePowerSetClockFrequency(ZX81.psp_cpu_clock, ZX81.psp_cpu_clock, ZX81.psp_cpu_clock/2);
}
