/*
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>

#include "global.h"
#include "psp_sdl.h"
#include "kbd.h"
#include "psp_kbd.h"
#include "psp_menu.h"
#include "psp_fmgr.h"
#include "psp_menu_kbd.h"
#include "psp_battery.h"

extern SDL_Surface *back_surface;

# define MENU_RENDER       0
# define MENU_COLOR        1
# define MENU_CLOCK        2
# define MENU_SPEED_LIMIT  3
# define MENU_ANALOG       4
# define MENU_SCREENSHOT   5

# define MENU_LOAD_SLOT    6
# define MENU_SAVE_SLOT    7
# define MENU_DEL_SLOT     8

# define MENU_LOAD_PRG     9
# define MENU_KEYBOARD    10

# define MENU_SAVE_CFG    11
# define MENU_RESET       12
# define MENU_BACK        13
# define MENU_EXIT        14

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  { 
    { "Render mode         :" },
    { "Color mode          :" },
    { "Clock frequency     :" },
    { "Speed limiter       :" },
    { "Swap Analog/Cursor  :" },
    { "Save Screenshot     :" },

    { "Load Slot" },
    { "Save Slot" },
    { "Delete Slot" },

    { "Load Program" },
    { "Keyboard" },
    { "Save Config" },

    { "Reset ZX81" },
    { "Back to ZX81" },
    { "Exit" }
  };

  static int cur_menu_id = MENU_LOAD_PRG;
  static int cur_slot    = 0;


  static int zx81_render_mode     = 0;
  static int zx81_speed_limiter   = 1;
  static int psp_reverse_analog   = 0;
  static int psp_cpu_clock        = 222;
  static int zx81_color_mode      = 0;

void 
psp_menu_display_save_name()
{
  char buffer[32];
  int Length;
  snprintf(buffer, 20, "Game: %s", ZX81.zx81_save_name);
  Length = strlen(buffer);
  psp_sdl_back2_print(460 - (8*Length), 25, buffer, PSP_MENU_TEXT2_COLOR);
}

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

void
psp_display_screen_menu_battery(void)
{
  char buffer[64];

  int Length;
  int color;

  snprintf(buffer, 50, " [%s] ", psp_get_battery_string());
  Length = strlen(buffer);

  if (psp_is_low_battery()) color = PSP_MENU_RED_COLOR;
  else                      color = PSP_MENU_GREEN_COLOR;

  psp_sdl_back2_print(240 - ((8*Length) / 2), 6, buffer, color);
}

static void 
psp_display_screen_menu(void)
{
  char buffer[64];
  int menu_id = 0;
  int slot_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  psp_sdl_blit_background();

  psp_sdl_draw_rectangle(10,10,459,249,PSP_MENU_BORDER_COLOR,0);
  psp_sdl_draw_rectangle(11,11,457,247,PSP_MENU_BORDER_COLOR,0);

  psp_sdl_back2_print( 30, 6, " Start+L+R: EXIT ", 
                     PSP_MENU_WARNING_COLOR,  PSP_MENU_BACKGROUND_COLOR);
  psp_display_screen_menu_battery();

  psp_sdl_back2_print( 370, 6, " R: RESET ", 
                     PSP_MENU_NOTE_COLOR,  PSP_MENU_BACKGROUND_COLOR);

  psp_sdl_back2_print(30, 254, " []: Cancel  O/X: Valid  SELECT: Back ", 
                     PSP_MENU_BORDER_COLOR);
  psp_sdl_back2_print(370, 254, " By Zx-81 ",
                     PSP_MENU_AUTHOR_COLOR);
  
  x      = 20;
  y      = 25;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = PSP_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = PSP_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = PSP_MENU_WARNING_COLOR;

    psp_sdl_back2_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_ANALOG) {
      if (psp_reverse_analog) strcpy(buffer,"yes");
      else                    strcpy(buffer,"no ");
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(210, y, buffer, color);
    } else
    if (menu_id == MENU_SPEED_LIMIT) {
      if (zx81_speed_limiter) strcpy(buffer,"yes");
      else                    strcpy(buffer,"no ");
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(210, y, buffer, color);
      if (zx81_speed_limiter == 0)  strcpy(buffer, "no");
      else sprintf(buffer, "%d fps", 60 / zx81_speed_limiter);

      string_fill_with_space(buffer, 7);
      psp_sdl_back2_print(210, y, buffer, color);
    } else
    if (menu_id == MENU_COLOR) {
      if (zx81_color_mode == ZX81_COLOR_BLUE)  strcpy(buffer, "blue");
      else 
      if (zx81_color_mode == ZX81_COLOR_WHITE) strcpy(buffer, "white");
      else 
      if (zx81_color_mode == ZX81_COLOR_BLACK) strcpy(buffer, "black");
      else                                     strcpy(buffer, "gray");
      string_fill_with_space(buffer, 6);
      psp_sdl_back2_print(210, y, buffer, color);

    } else
    if (menu_id == MENU_RENDER) {
      if (zx81_render_mode == ZX81_RENDER_NORMAL    ) strcpy(buffer, "normal");
      else 
      if (zx81_render_mode == ZX81_RENDER_FIT       ) strcpy(buffer, "fit");
      else
      if (zx81_render_mode == ZX81_RENDER_FIT_WIDTH ) strcpy(buffer, "fit width");
      else                                            strcpy(buffer, "max");
      string_fill_with_space(buffer, 13);
      psp_sdl_back2_print(210, y, buffer, color);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer,"%d", psp_cpu_clock);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(210, y, buffer, color);
    } else
    if (menu_id == MENU_SCREENSHOT) {
      sprintf(buffer,"%d", ZX81.psp_screenshot_id);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(210, y, buffer, color);
      y += y_step;
    } else
    if (menu_id == MENU_DEL_SLOT) {
      y += 2*y_step;
    } else
    if (menu_id == MENU_KEYBOARD) {
      y += y_step;
    } else
    if (menu_id == MENU_BACK) {
      y += y_step;
    }

    y += y_step;
  }

  y_step = 10;
  y      = 95;

  for (slot_id = 0; slot_id < ZX81_MAX_SAVE_STATE; slot_id++) {
    if (slot_id == cur_slot) color = PSP_MENU_SEL2_COLOR;
    else                     color = PSP_MENU_TEXT_COLOR;

    if (ZX81.zx81_save_used[slot_id]) sprintf(buffer, "%d - used" , slot_id + 1);
    else                              sprintf(buffer, "%d - empty", slot_id + 1);
    psp_sdl_back2_print(160, y, buffer, color);

    y += y_step;
  }

  psp_menu_display_save_name();
}

static void
psp_main_menu_reset(void)
{
  /* Reset ! */
  psp_display_screen_menu();
  psp_sdl_back2_print(270,  80, "Reset ZX81 !", 
                     PSP_MENU_WARNING_COLOR);
  psp_sdl_flip();
  emulator_reset();
  sleep(1);
}

static void
psp_main_menu_clock(int step)
{
  if (step > 0) {
    if (psp_cpu_clock == 222) {
      psp_cpu_clock = 300;
    } else if (psp_cpu_clock == 300) {
      psp_cpu_clock = 333;
    } else {
      psp_cpu_clock = 222;
    }
  } else {
    if (psp_cpu_clock == 222) {
      psp_cpu_clock = 333;
    } else if (psp_cpu_clock == 333) {
      psp_cpu_clock = 300;
    } else {
      psp_cpu_clock = 222;
    }
  }
}

static void
psp_main_menu_render(int step)
{
# if 0
  if (step > 0) {
    if (zx81_render_mode < ZX81_LAST_RENDER) zx81_render_mode++;
    else                                   zx81_render_mode = 0;
  } else {
    if (zx81_render_mode > 0) zx81_render_mode--;
    else                     zx81_render_mode = ZX81_LAST_RENDER;
  }
# else
  /* TO_BE_DONE ! */
  zx81_render_mode = ZX81_RENDER_NORMAL;
# endif
}

static void
psp_main_menu_color(int step)
{
  if (step > 0) {
    if (zx81_color_mode < ZX81_LAST_COLOR) zx81_color_mode++;
    else                                   zx81_color_mode = 0;
  } else {
    if (zx81_color_mode > 0) zx81_color_mode--;
    else                     zx81_color_mode = ZX81_LAST_COLOR;
  }
}

static void
psp_main_menu_limiter(int step)
{
  if (step > 0) {
    if (zx81_speed_limiter < 3) zx81_speed_limiter++;
    else                        zx81_speed_limiter = 0;
  } else {
    if (zx81_speed_limiter > 0) zx81_speed_limiter--;
    else                        zx81_speed_limiter = 3;
  }
}

static void
psp_main_menu_load(int format)
{
  int ret;

  ret = psp_fmgr_menu(format);
  if (ret ==  1) /* load OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "File loaded !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Can't load file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_load_slot()
{
  int error;

  if (! ZX81.zx81_save_used[cur_slot]) {

    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Slot is empty !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);

    return;
  }

  error = zx81_snapshot_load_slot(cur_slot);

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "File loaded !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Can't load file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_save_slot()
{
  int error;
  error = zx81_snapshot_save_slot(cur_slot);

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "File saved !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Can't save file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_del_slot()
{
  int error;

  if (! ZX81.zx81_save_used[cur_slot]) {

    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Slot is empty !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);

    return;
  }

  error = zx81_snapshot_del_slot(cur_slot);

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "File deleted !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Can't delete file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_cur_slot(int step)
{
  if (step == 1) {
    cur_slot++; if (cur_slot >= ZX81_MAX_SAVE_STATE) cur_slot = 0;
  } else if (step == -1) {
    cur_slot--; if (cur_slot < 0) cur_slot = ZX81_MAX_SAVE_STATE - 1;
  }
}

static void
psp_main_menu_screenshot(void)
{
  psp_screenshot_mode = 10;
}

static void
psp_main_menu_validate(void)
{
  /* Validate */
  ZX81.zx81_render_mode    = zx81_render_mode;
  ZX81.zx81_color_mode     = zx81_color_mode;
  ZX81.zx81_speed_limiter  = zx81_speed_limiter;
  ZX81.psp_cpu_clock       = psp_cpu_clock;
  ZX81.psp_reverse_analog  = psp_reverse_analog;

  scePowerSetClockFrequency(ZX81.psp_cpu_clock, ZX81.psp_cpu_clock, ZX81.psp_cpu_clock/2);
}

static void
psp_main_menu_save_config()
{
  int error;

  psp_main_menu_validate();

  error = zx81_save_configuration();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "File saved !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  80, "Can't save file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

int
psp_main_menu_exit(void)
{
  SceCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back2_print(270,  80, "press X to confirm !", PSP_MENU_WARNING_COLOR);
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    sceCtrlReadBufferPositive(&c, 1);

    if (c.Buttons & PSP_CTRL_CROSS) psp_sdl_exit(0);

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();
}

int 
psp_main_menu(void)
{
  SceCtrlData c;
  long        new_pad;
  long        old_pad;
  int         last_time;
  int         end_menu;

  psp_kbd_wait_no_button();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  zx81_render_mode    = ZX81.zx81_render_mode;
  zx81_color_mode     = ZX81.zx81_color_mode;
  zx81_speed_limiter  = ZX81.zx81_speed_limiter;
  psp_cpu_clock       = ZX81.psp_cpu_clock;
  psp_reverse_analog  = ZX81.psp_reverse_analog;

  while (! end_menu)
  {
    psp_display_screen_menu();
    psp_sdl_flip();

    while (1)
    {
      sceCtrlReadBufferPositive(&c, 1);
      if (c.Buttons) break;
    }

    new_pad = c.Buttons;

    if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
      last_time = c.TimeStamp;
      old_pad = new_pad;

    } else continue;

    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
        (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((c.Buttons & PSP_CTRL_RTRIGGER) == PSP_CTRL_RTRIGGER) {
      psp_main_menu_reset();
      end_menu = 1;
    } else
    if ((new_pad & PSP_CTRL_CROSS ) || 
        (new_pad & PSP_CTRL_CIRCLE) || 
        (new_pad & PSP_CTRL_RIGHT ) ||
        (new_pad & PSP_CTRL_LEFT  )) 
    {
      int step;

      if (new_pad & PSP_CTRL_LEFT)  step = -1;
      else 
      if (new_pad & PSP_CTRL_RIGHT) step =  1;
      else                          step =  0;

      switch (cur_menu_id ) 
      {
        case MENU_ANALOG    : psp_reverse_analog = ! psp_reverse_analog;
        break;              
        case MENU_SPEED_LIMIT : psp_main_menu_limiter( step );
        break;              
        case MENU_COLOR     : psp_main_menu_color( step );
        break;              
        case MENU_RENDER    : psp_main_menu_render( step );
        break;              
        case MENU_CLOCK     : psp_main_menu_clock( step );
        break;
        case MENU_LOAD_SLOT : if (step) psp_main_menu_cur_slot(step);
                              else      psp_main_menu_load_slot();
        break;
        case MENU_SAVE_SLOT : if (step) psp_main_menu_cur_slot(step);
                              else      psp_main_menu_save_slot();
        break;
        case MENU_DEL_SLOT  : if (step) psp_main_menu_cur_slot(step);
                              else      psp_main_menu_del_slot();
        break;
        case MENU_LOAD_PRG   : psp_main_menu_load(FMGR_FORMAT_PRG);
                               old_pad = new_pad = 0;
        break;              
        case MENU_KEYBOARD   : psp_keyboard_menu();
                               old_pad = new_pad = 0;
        break;
        case MENU_SCREENSHOT : psp_main_menu_screenshot();
                               end_menu = 1;
        break;              

        case MENU_SAVE_CFG  : psp_main_menu_save_config();
        break;
        case MENU_RESET     : psp_main_menu_reset();
                              end_menu = 1;
        break;

        case MENU_BACK      : end_menu = 1;
        break;

        case MENU_EXIT      : psp_main_menu_exit();
        break;
      }

    } else
    if(new_pad & PSP_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & PSP_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & PSP_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if(new_pad & PSP_CTRL_SELECT) {
      /* Back to ZX81 */
      end_menu = 1;
    }
  }
 
  if (end_menu > 0) {
    psp_main_menu_validate();
  }

  psp_kbd_wait_no_button();

  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();
  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();

  return 1;
}

